@@ -18,18 +18,21 @@
 
 
 /* Zktor is a word that does not mean anything and is difficult to pronounce. */
-
 /* I apologize for horrible coding. */
 
 
 #include <math.h>
+#include <stdlib.h>
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
 #include <gdk/gdkkeysyms.h>
 #include <GL/gl.h>
 #include <GL/glu.h>
+
 #include <gtkgl/gtkglarea.h>
 
+
+
 typedef struct {
   int   state; /* zero state means dead */
   float timer;
@@ -43,21 +46,27 @@ typedef struct {
 
 double game_time;
 double game_tick;
-
+int draw_fast = 0;
 
 int wave_cnt;
 double wave_time;
+double vortex_time;
 
 int control_speed;
 int control_spin;
 int control_fire;
 
+int score = 0;
+int highscore = 0;
+
 Entity player;
 Entity enemy[10];
 Entity vortex[10];
 Entity p_bullet[20];
 Entity e_bullet[20];
+Entity particle[60];
 
+GLuint fontbase = 0;
 
 float rnd()
 {
@@ -95,11 +104,15 @@ void game_init()
 
   wave_cnt = 0;
   wave_time = game_time + 5; /* give 5 secs before start of waves */
+  vortex_time = game_time + 3; /* give 3 secs before 1st vortex */
+
 
   control_speed = 0;
   control_spin  = 0;
   control_fire  = 0;
 
+  score = 0;
+
   player.state = 1;
   player.timer = game_time;
   player.pos_x = 0;
@@ -214,6 +227,26 @@ void game_play()
 	}
       }
     }
+
+    /* if speed control is pressed create more particles */
+    if (control_speed) {
+      for (i=0; i<sizeof(particle)/sizeof(Entity); i++) {
+	if (!particle[i].state) {
+	  float spread = rnd()*15 - 7.5;
+	  particle[i].state = 1;
+	  particle[i].timer = game_time + rnd()*.8 + .2;
+	  particle[i].pos_x = player.pos_x - player.radius*dx;
+	  particle[i].pos_y = player.pos_y - player.radius*dy;
+	  particle[i].vel_x = player.vel_x - dx*25 - dy * spread;
+	  particle[i].vel_y = player.vel_y - dy*25 + dx * spread;
+	  particle[i].dir   = player.dir;
+	  particle[i].radius= .1;
+	  break;
+	}
+      }
+    }
+
+
   } else {
     /* enemies continue to chase player position even if player is dead
        - make it center of screen if player is dead */
@@ -300,7 +333,7 @@ void game_play()
       /* kill this vortex, time is up */
       if (vortex[i].timer < game_time) {
 	vortex[i].state = 0;
-	break;
+	continue;
       }
       /* move */
       vortex[i].dir   += 60  * game_tick;
@@ -339,20 +372,18 @@ void game_play()
 	}
       }
 
-    } else {
-      /* vortices form from bubbling vacuum energy */
-      if (rnd() < 0.5*game_tick && rnd() < 0.5*game_tick) {
-	do {
-	  vortex[i].pos_x  = 200*rnd()-100;
-	  vortex[i].pos_y  = 200*rnd()-100;
-	  vortex[i].radius = rnd()*10 + 10;
-	} while (collision(&vortex[i], &player));
-	vortex[i].state = 1;
-	vortex[i].timer = game_time + rnd()*15 + 5;
-	do vortex[i].vel_x = rnd()*30 - 15; while (fabs(vortex[i].vel_x) < 10);
-	do vortex[i].vel_y = rnd()*30 - 15; while (fabs(vortex[i].vel_y) < 10);
-	vortex[i].dir = 0;
-      }
+    } else if (vortex_time < game_time) {
+      vortex_time = game_time + rnd()*10 + 5;
+      do {
+	vortex[i].pos_x  = 200*rnd()-100;
+	vortex[i].pos_y  = 200*rnd()-100;
+	vortex[i].radius = rnd()*10 + 10;
+      } while (collision(&vortex[i], &player));
+      vortex[i].state = 1;
+      vortex[i].timer = game_time + rnd()*15 + 5;
+      do vortex[i].vel_x = rnd()*30 - 15; while (fabs(vortex[i].vel_x) < 10);
+      do vortex[i].vel_y = rnd()*30 - 15; while (fabs(vortex[i].vel_y) < 10);
+      vortex[i].dir = 0;      
     }
   }
 
@@ -372,8 +403,11 @@ void game_play()
 	p_bullet[i].state = 0;
       }
       /* collision to enemy. kills bullet and kills enemy */
+      /* and adds 100 to score */
       for (j=0; j<sizeof(enemy)/sizeof(Entity); j++) {
 	if (collision(&p_bullet[i],&enemy[j])) {
+	  score += 100;
+	  if (score > highscore) highscore = score;
 	  p_bullet[i].state = 0;
 	  enemy[j].state = 0;
 	}
@@ -418,12 +452,39 @@ void game_play()
     }
   }
 
+  /* particle */
+  for (i=0; i<sizeof(particle)/sizeof(Entity); i++) {
+    if (particle[i].state) {
+      /* time elapsed, kill particle */
+      if (particle[i].timer < game_time) {
+	particle[i].state = 0;
+	continue;
+      }
+      /* move */
+      particle[i].pos_x += particle[i].vel_x * game_tick;
+      particle[i].pos_y += particle[i].vel_y * game_tick;
+    }
+  }
+
 }
 
 void game_render()
 {
   int i;
 
+  /* drawmode */
+  if (draw_fast) {
+    glDisable(GL_LINE_SMOOTH);
+    glDisable(GL_POINT_SMOOTH);
+    glDisable(GL_BLEND);
+  } else {
+    glEnable(GL_LINE_SMOOTH);
+    glEnable(GL_POINT_SMOOTH);
+    glEnable(GL_BLEND);
+    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+  }
+
+
   /* view */
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
@@ -514,7 +575,6 @@ void game_render()
   /* vortex */
   for (i=0; i<sizeof(vortex)/sizeof(Entity); i++) {
     if (vortex[i].state) {
-      float angle;
       
       glPushMatrix();
       glTranslatef(vortex[i].pos_x, vortex[i].pos_y, 0);
@@ -571,6 +631,26 @@ void game_render()
     }
   }
 
+  /* particles */
+  glColor3f(.5,.7,1);
+  glBegin(GL_POINTS);
+  for (i=0; i<sizeof(particle)/sizeof(Entity); i++) {
+    if (particle[i].state)
+      glVertex2f(particle[i].pos_x, particle[i].pos_y);
+  }
+  glEnd();  
+
+  /* textual info */
+  if (fontbase) {
+    char s[200];
+    g_snprintf(s, sizeof(s), "wave %d score %d highscore %d", wave_cnt, score, highscore);
+    
+    glColor3f(.8,.8,.8);
+    glRasterPos2f(-90, 90);
+    glListBase(fontbase);
+    glCallLists(strlen(s), GL_UNSIGNED_BYTE, s);
+
+  }
 }
 
 
@@ -584,7 +664,20 @@ gint init(GtkWidget *widget)
 {
   /* OpenGL functions can be called only if begingl returns true */
   if (gtk_gl_area_begingl(GTK_GL_AREA(widget))) {
+    GdkFont *font;
+
+    /* set viewport */
     glViewport(0,0, widget->allocation.width, widget->allocation.height);
+
+    /* generate font display lists */
+    font = gdk_font_load("-adobe-helvetica-medium-r-normal--*-120-*-*-*-*-*-*");
+    if (font) {
+      fontbase = glGenLists( 128 );
+      gdk_gl_use_gdk_font(font, 0, 128, fontbase);
+      gdk_font_unref(font);
+    }
+
+
     /* end opengl calls by calling endgl */
     gtk_gl_area_endgl(GTK_GL_AREA(widget));
   }
@@ -632,44 +725,56 @@ gint reshape(GtkWidget *widget, GdkEventConfigure *event)
 }
 
 
-gint key_event(GtkWidget *widget, GdkEventKey *event)
+gint key_press_event(GtkWidget *widget, GdkEventKey *event)
 {
-  if (event->type == GDK_KEY_PRESS) {
-    switch (event->keyval) {
-    case GDK_Left:
-      control_spin = 1;
-      break;
-    case GDK_Right:
-      control_spin = -1;
-      break;
-    case GDK_space:
-    case GDK_Up:
-      control_fire = 1;
-      break;
-    case GDK_Down:
-      control_speed = 1;
-      break;
-    case GDK_r:
-      game_init();
-      break;
-    }
-  } else {
-    switch (event->keyval) {
-    case GDK_Left:
-      if (control_spin == 1) control_spin = 0;
-      break;
-    case GDK_Right:
-      if (control_spin == -1) control_spin = 0;
-      break;
-    case GDK_space:
-    case GDK_Up:
-      control_fire = 0;
-      break;
-    case GDK_Down:
-      control_speed = 0;
-      break;
-    }
+  switch (event->keyval) {
+  case GDK_Left:
+    control_spin = 1;
+    break;
+  case GDK_Right:
+    control_spin = -1;
+    break;
+  case GDK_Up:
+  case GDK_space:
+    control_fire = 1;
+    break;
+  case GDK_Down:
+    control_speed = 1;
+    break;
+  case GDK_r:
+    game_init();
+    break;
+  case GDK_q:
+    gtk_main_quit();
+    break;
+  case GDK_d:
+    draw_fast = (!draw_fast);
+    break;
   }
+  /* prevent the default handler from being run */
+  gtk_signal_emit_stop_by_name(GTK_OBJECT(widget),"key_press_event");
+  return TRUE;
+}
+
+gint key_release_event(GtkWidget *widget, GdkEventKey *event)
+{
+  switch (event->keyval) {
+  case GDK_Left:
+    if (control_spin == 1) control_spin = 0;
+    break;
+  case GDK_Right:
+    if (control_spin == -1) control_spin = 0;
+    break;
+  case GDK_Up:
+  case GDK_space:
+    control_fire = 0;
+    break;
+  case GDK_Down:
+    control_speed = 0;
+    break;
+  }
+  /* prevent the default handler from being run */
+  gtk_signal_emit_stop_by_name(GTK_OBJECT(widget),"key_release_event");
   return TRUE;
 }
 
@@ -681,7 +786,6 @@ gint animate(GtkWidget *glarea)
 }
 
 
-
 int main(int argc, char **argv)
 {
   GtkWidget *window,*vbox,*logo,*glarea;
@@ -692,7 +796,7 @@ int main(int argc, char **argv)
      GDK_GL_NONE. See glXChooseVisual manpage for further
      explanation.
   */
-  gint attrlist[] = {
+  int attrlist[] = {
     GDK_GL_RGBA,
     GDK_GL_DOUBLEBUFFER,
     GDK_GL_NONE
@@ -744,7 +848,7 @@ int main(int argc, char **argv)
   /* set minimum size */
   /*  gtk_widget_set_usize(GTK_WIDGET(glarea), 200,200); */
   /* set default size */
-  gtk_gl_area_size(GTK_GL_AREA(glarea), 400,400);
+  gtk_gl_area_size(GTK_GL_AREA(glarea), 640,400);
 
   
   /* Connect signal handlers */
@@ -759,9 +863,9 @@ int main(int argc, char **argv)
 		     GTK_SIGNAL_FUNC(init), NULL);
   /* Capture keypress events */
   gtk_signal_connect(GTK_OBJECT(glarea), "key_press_event",
-		     GTK_SIGNAL_FUNC(key_event), NULL);
+		     GTK_SIGNAL_FUNC(key_press_event), NULL);
   gtk_signal_connect(GTK_OBJECT(glarea), "key_release_event",
-		     GTK_SIGNAL_FUNC(key_event), NULL);
+		     GTK_SIGNAL_FUNC(key_release_event), NULL);
 
   /* construct widget hierarchy  */
   gtk_container_add(GTK_CONTAINER(window),GTK_WIDGET(vbox));
@@ -776,7 +880,6 @@ int main(int argc, char **argv)
   gtk_widget_show(GTK_WIDGET(vbox));
   gtk_widget_show(window);
 
-
   /* set focus to glarea widget */
   GTK_WIDGET_SET_FLAGS(glarea, GTK_CAN_FOCUS);
   gtk_widget_grab_focus(GTK_WIDGET(glarea));
@@ -787,5 +890,6 @@ int main(int argc, char **argv)
   game_init();
   gtk_main();
 
+
   return 0;
 }
